// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       CB_Lighting_Static.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__CB_LIGHTING_STATIC
#define TINMAN__CB_LIGHTING_STATIC

#include "../GpuX.hlsl"

// -------------------------------------------------------------------------------------------------
// Constant Buffers
// -------------------------------------------------------------------------------------------------

/*!
 * @brief A constant buffer for use with 'LightingParameters'.
 */
cbuffer CB_Lighting_Static
{
  /*!
   * @brief The constant data field g_lightSpace.
   */
  float3x3 g_lightSpace;

  /*!
   * @brief The constant data field g_edgeFixup.
   */
  float g_edgeFixup;

  /*!
   * @brief The constant data field g_lightVector.
   */
  float3 g_lightVector;

  /*!
   * @brief The constant data field g_horizon.
   */
  float g_horizon;

  /*!
   * @brief The constant data field g_mipmap.
   */
  float3 g_mipmap;

  /*!
   * @brief The constant data field g_environmentScale.
   */
  float g_environmentScale;

  /*!
   * @brief The constant data field g_scale.
   */
  float4 g_scale;

  /*!
   * @brief The constant data field g_coefficient.
   */
  float3 g_coefficient;

  /*!
   * @brief The constant data field g_fogDistance.
   */
  float g_fogDistance;

  /*!
   * @brief The constant data field g_unlitShade.
   */
  float g_unlitShade;
};

#endif
