// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Model.PS_Pixel.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__MODEL__MODEL_PS_PIXEL
#define TINMAN__MODEL__MODEL_PS_PIXEL

#include "../../GpuX.hlsl"
#include "../PS_Out_TargetDepthLess.hlsl"
#include "Model.hlsl"
#include "VS_Out.hlsl"

// -------------------------------------------------------------------------------------------------
// Pixel Shader 'Model.PS_Pixel'
// -------------------------------------------------------------------------------------------------

/*!
 * @brief The pixel shader for opaque geometry with per-pixel frequency.
 *
 * @param data The shader input.
 * @param face The shader input.
 *
 * @returns The shader output.
 */
PS_Out_TargetDepthLess main(
    centroid float4 data_Color SEMANTIC_COLOR(0),
    centroid float4 data_Coords SEMANTIC_TEXCOORD(1),
    float4 data_Plain SEMANTIC_TEXCOORD(3),
    centroid float4 data_Position SEMANTIC_SYSTEM_POSITION(0),
    centroid float4 data_Shadow SEMANTIC_TEXCOORD(4),
    centroid float3 data_Bitangent SEMANTIC_TANGENT(0),
    centroid float3 data_Normal SEMANTIC_NORMAL(1),
    centroid float3 data_Tangent SEMANTIC_TANGENT(1),
    centroid noperspective float2 data_Clip SEMANTIC_TEXCOORD(0),
    centroid float data_Occlusion SEMANTIC_TEXCOORD(2), FACE face)
{
  VS_Out data;

  data.Color = data_Color;
  data.Coords = data_Coords;
  data.Plain = data_Plain;
  data.Position = data_Position;
  data.Shadow = data_Shadow;
  data.Bitangent = data_Bitangent;
  data.Normal = data_Normal;
  data.Tangent = data_Tangent;
  data.Clip = data_Clip;
  data.Occlusion = data_Occlusion;

  PS_Out_TargetDepthLess result;

  result = ModelPixel(data, face);

  return result;
}

#endif
