// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Model.VS_InstanceShadow.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__MODEL__MODEL_VS_INSTANCESHADOW
#define TINMAN__MODEL__MODEL_VS_INSTANCESHADOW

#include "../../GpuX.hlsl"
#include "../Camera.hlsl"
#include "Model.hlsl"
#include "VS_In_InstanceShadow.hlsl"
#include "VS_In_Shadow.hlsl"
#include "VS_Out_Shadow.hlsl"

// -------------------------------------------------------------------------------------------------
// Vertex Shader 'Model.VS_InstanceShadow'
// -------------------------------------------------------------------------------------------------

/*!
 * @brief The vertex shader for instanced geometry with 'ModelRendererMode.Shadow'.
 *
 * @param vertex   The shader input.
 * @param instance The shader input.
 *
 * @returns The shader output.
 */
VS_Out_Shadow main(VS_In_Shadow vertex, VS_In_InstanceShadow instance)
{
  VS_Out_Shadow result;
  float3x3 m;
  float t;

  // Apply model transformation.

  m = float3x3(instance.Matrix0, instance.Matrix1, cross(instance.Matrix0, instance.Matrix1));
  t = sign(instance.Intensity);
  vertex.Position *= instance.Scale;
  vertex.PositionExtra *= instance.Scale;
  vertex.Position = mul(vertex.Position, m);
  vertex.PositionExtra = mul(vertex.PositionExtra, m) + instance.Position * max(0.0, -t);

  ModelTransform(vertex.Position, vertex.PositionExtra);

  vertex.PositionExtra += instance.Position * max(0.0, t);
  vertex.Position = ModelPosition(false, vertex.Position, vertex.PositionExtra);

  // Compute output values.

  result.Color = float4(1, 1, 1, VertexAlpha(vertex.Color));
  result.Coords = float4(vertex.Coords[0], vertex.Coords[1]);
  result.Distance = length(vertex.Position);
  result.Position = ProjectPositionShadow(vertex.Position, result.Depth);

  return result;
}

#endif
