// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       CB_Texture_Static.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__CB_TEXTURE_STATIC
#define TINMAN__TERRAIN__CB_TEXTURE_STATIC

#include "../../GpuX.hlsl"
#include "SR_Terrain.hlsl"

// -------------------------------------------------------------------------------------------------
// Constant Buffers
// -------------------------------------------------------------------------------------------------

/*!
 * @brief A constant buffer for use with 'TerrainTextureParameters' and 'TextureTileParameters'.
 */
cbuffer CB_Texture_Static
{
  /*!
   * @brief The constant data field g_materialAlbedo.
   *
   * @see 'TerrainTextureParameters'
   */
  float4 g_materialAlbedo;

  /*!
   * @brief The constant data field g_materialEmissive.
   *
   * @see 'TerrainTextureParameters'
   */
  float4 g_materialEmissive;

  /*!
   * @brief The constant data field g_materialGloss.
   *
   * @see 'TerrainTextureParameters'
   */
  float4 g_materialGloss;

  /*!
   * @brief The constant data field g_materialNormal.
   *
   * @see 'TerrainTextureParameters'
   */
  float4 g_materialNormal;

  /*!
   * @brief The constant data field g_materialOcclusion.
   *
   * @see 'TerrainTextureParameters'
   */
  float4 g_materialOcclusion;

  /*!
   * @brief The constant data field g_materialOpacity.
   *
   * @see 'TerrainTextureParameters'
   */
  float4 g_materialOpacity;

  /*!
   * @brief The constant data field g_materialReflectivity.
   *
   * @see 'TerrainTextureParameters'
   */
  float4 g_materialReflectivity;

  /*!
   * @brief The constant data field g_materialEmissiveColor.
   *
   * @see 'TerrainTextureParameters'
   */
  float3 g_materialEmissiveColor;

  /*!
   * @brief The constant data field g_materialReflectivityColor.
   *
   * @see 'TerrainTextureParameters'
   */
  float3 g_materialReflectivityColor;

  /*!
   * @brief The constant data field g_materialEmissiveFade.
   *
   * @see 'TerrainTextureParameters'
   */
  float2 g_materialEmissiveFade;

  /*!
   * @brief The constant data field g_terrainData.
   *
   * @see 'TerrainTextureParameters'
   */
  float g_terrainData;

  /*!
   * @brief The constant data field g_tileGamma.
   *
   * @see 'TextureTileParameters'
   */
  float g_tileGamma[TextureCount * 2];
};

#endif
