// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       CB_Texture_Volatile.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__CB_TEXTURE_VOLATILE
#define TINMAN__TERRAIN__CB_TEXTURE_VOLATILE

#include "../../GpuX.hlsl"
#include "SR_Terrain.hlsl"

// -------------------------------------------------------------------------------------------------
// Constant Buffers
// -------------------------------------------------------------------------------------------------

/*!
 * @brief A constant buffer for use with 'TexturingBatchParameters' and 'TextureTileParameters'
 */
cbuffer CB_Texture_Volatile
{
  /*!
   * @brief The constant data field g_tileBlend0.
   *
   * @see 'TexturingBatchParameters'
   */
  float4 g_tileBlend0[TextureCount];

  /*!
   * @brief The constant data field g_tileBlend1.
   *
   * @see 'TexturingBatchParameters'
   */
  float4 g_tileBlend1[TextureCount];

  /*!
   * @brief The constant data field g_tileBounds.
   *
   * @see 'TextureTileParameters'
   */
  float4 g_tileBounds[TextureCount * 2];

  /*!
   * @brief The constant data field g_tileOffset.
   *
   * @see 'TextureTileParameters'
   */
  float4 g_tileOffset[TextureCount * 2];

  /*!
   * @brief The constant data field g_tileU.
   *
   * @see 'TextureTileParameters'
   */
  float3 g_tileU[TextureCount * 2];

  /*!
   * @brief The constant data field g_tileV.
   *
   * @see 'TextureTileParameters'
   */
  float3 g_tileV[TextureCount * 2];

  /*!
   * @brief The constant data field g_tileBlendFactor.
   *
   * @see 'TexturingBatchParameters'
   */
  float g_tileBlendFactor[TextureCount];

  /*!
   * @brief The constant data field g_tileSlice.
   *
   * @see 'TextureTileParameters'
   */
  int g_tileSlice[TextureCount * 2];
};

#endif
