// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Terrain.TV_Material.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__TERRAIN_TV_MATERIAL
#define TINMAN__TERRAIN__TERRAIN_TV_MATERIAL

#include "../../GpuX.hlsl"
#include "../TS_Patch.hlsl"
#include "GS_In_Material.hlsl"
#include "PS_In_Material.hlsl"
#include "Terrain.hlsl"

// -------------------------------------------------------------------------------------------------
// Tessellate Vertex Shader 'Terrain.TV_Material'
// -------------------------------------------------------------------------------------------------

/*!
 * @brief The tessellate vertex shader for 'TerrainParameters.PassMaterial' of
 *        'TerrainFeatureLevel.DX11'.
 *
 * @param data   The shader input.
 * @param points The shader input.
 * @param domain The shader input.
 *
 * @returns The shader output.
 */
[domain("tri")]
PS_In_Material main(TS_Patch patch, const OutputPatch<GS_In_Material,3> points, float3 domain :
SV_DomainLocation)
{
  PS_In_Material result;
  GS_In_Material mixed;

  mixed = MaterialMix3(points[0], points[1], points[2], domain);
  result = MaterialCompute(mixed, ComputeDisplacementVarying(mixed.Fixed, mixed.WeightsA, true));

  return result;
}

#endif
