// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Terrain.VS_Background.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__TERRAIN_VS_BACKGROUND
#define TINMAN__TERRAIN__TERRAIN_VS_BACKGROUND

#include "../../GpuX.hlsl"
#include "../CB_Camera_Static.hlsl"
#include "../Util.hlsl"
#include "PS_In_Background.hlsl"
#include "SR_Terrain.hlsl"
#include "Terrain.hlsl"
#include "VS_In.hlsl"

// -------------------------------------------------------------------------------------------------
// Vertex Shader 'Terrain.VS_Background'
// -------------------------------------------------------------------------------------------------

/*!
 * @brief The vertex shader for 'TerrainParameters.PassTextureDome'.
 *
 * @param data The shader input.
 *
 * @returns The shader output.
 */
PS_In_Background main(VS_In data)
{
  PS_In_Background result;
  float4x4 m0;
  float4x4 m1;
  float3 p0;
  float3 p1;
  float4 v00;
  float4 v10;
  float4 v01;
  float4 v11;

  m0 = g_viewProjectionMatrix[0];
  m1 = g_viewProjectionMatrix[1];
  p0 = data.Position;
  p1 = data.PositionDelta;
  v00 = mul(m0, float4(p0, 1));
  v10 = mul(m1, float4(p0, 0));
  v01 = mul(m0, float4(p1, 1));
  v11 = mul(m1, float4(p1, 0));
  result.Position = v00 + (v10 + v01) + v11;
  result.Position.z = result.Position.w;
  result.Color = FromSrgb(data.Color);
  result.Distance = length(p0);

  for (int i = 0; i < TextureCount; i++)
  {
    float2 xy;
    float2 zw;

    TextureCoords(i, data.CoordsHigh, data.CoordsLow, xy, zw);

    result.TexCoords[i] = float4(xy, zw);
  }

  return result;
}

#endif
