// -------------------------------------------------------------------------------------------------
//          _____          _______                                   ____   _____
//      ,-:` \;',`'-,     |__  _(_)                                 |___ \ |  __ \
//    .'-;_,;  ':-;_,'.      | | _  _ __   _ __ ___    __ _  _ __     __) || |  | |
//   /;   '/    ,  _`.-\     | || || '_ \ | '_ ` _ \  / _` || '_ \   |__ < | |  | |
//  | '`. (`     /` ` \`|    | || || | | || | | | | || (_| || | | |  ___) || |__| |
//  |:.  `\`-.   \_   / |    |_||_||_| |_||_| |_| |_| \__,_||_| |_| |____/ |_____/
//  |     (   `,  .`\ ;'|
//   \     | .'     `-'/      Realtime terrain processing, rendering and analysis
//    `.   ;/        .'
//      `'-._____.-'`             http://www.tinman3d.com - info@tinman3d.com
// -------------------------------------------------------------------------------------------------
// FILE:       Triangulate.CS_TriangleTerminal.hlsl
// -------------------------------------------------------------------------------------------------
// COPYRIGHT:  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// AUTHOR(s):  Matthias Englert, me@tinman3d.com
// -------------------------------------------------------------------------------------------------
// DISCLAIMER: This source  code is  provided "as  is" and  without warranties  as to performance or
//             merchantability. The  author and/or  distributors of  this source  code may have made
//             statements about this source code.  Any such statements do not  constitute warranties
//             and shall not be  relied on by  the user in deciding whether to use this source code.
//             This source code is provided without any expressed or implied warranties  whatsoever.
//             Because of the diversity of conditions and hardware under which this source code  may
//             be used,  no warranty  of fitness  for a  particular purpose  is offered. The user is
//             advised to test the source code thoroughly before relying on it. The user must assume
//             the entire risk of using the source code.
// -------------------------------------------------------------------------------------------------

#ifndef TINMAN__TERRAIN__TRIANGULATE_CS_TRIANGLETERMINAL
#define TINMAN__TERRAIN__TRIANGULATE_CS_TRIANGLETERMINAL

#include "../../GpuX.hlsl"
#include "CB_Compute_Static.hlsl"
#include "SR_Triangulate.hlsl"
#include "Triangulate.hlsl"

// -------------------------------------------------------------------------------------------------
// Compute Shader 'Triangulate.CS_TriangleTerminal'
// -------------------------------------------------------------------------------------------------

/*!
 * @brief The compute shader for decoding 'TriangleMode.TriangleTerminal'.
 *
 * @param groupId  The shader input.
 * @param threadId The shader input.
 */
[numthreads(119,1,1)]
void main(int3 groupId : SV_GroupID, int3 threadId : SV_GroupThreadID)
{
  int sliceCount;
  int groupCount;
  int triangleCount;
  int outputLast;

  sliceCount = g_count - 1;
  groupCount = clamp(sliceCount, 1, ComputeGroups);

  // Divide the output buffer into equally sized parts. Then, each thread group fills one part.

  triangleCount = g_capacity / 3;
  outputLast = groupId.x * triangleCount / groupCount * 3;

  if (threadId.x == 0)
  {
    computeOutputIdx = outputLast;
    computeOutputEnd = (groupId.x + 1) * triangleCount / groupCount * 3;
  }

  // Run through triangulation slices, interweave by group ID.

  for (int slice = groupId.x; slice < sliceCount; slice += groupCount)
  {
    int from;
    int to;

    from = g_slices[slice];
    to = g_slices[slice + 1];

    // Run through terminal triangle codes, interweave by thread ID.

    gxSync();
    Compute_TerminalTriangle(from + threadId.x, to, ComputeThreads);
    gxSync();

    // Write resulting indirect draw command arguments.

    if (threadId.x == 0)
    {
      int temp;

      temp = min(computeOutputIdx, computeOutputEnd);

      Compute_OutputDraw(slice, outputLast, temp - outputLast);

      outputLast = temp;
    }
  }
}

#endif
